==> opengtl_0.9.12.orig.tar.bz2: OpenGTL-0.9.12/OpenCTL/OpenCTL/Module.cpp <==
/*
 *  Copyright (c) 2007 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "Module.h"

#include <sstream>
#include <fstream>

#include "GTLCore/ErrorMessage.h"
#include "GTLCore/Function.h"
#include "GTLCore/Parameter.h"
#include "GTLCore/Macros_p.h"
#include "GTLCore/VirtualMachine_p.h"
#include "GTLCore/ModuleData_p.h"
#include "GTLCore/Type.h"

#include "Compiler.h"
#include "Debug.h"

using namespace OpenCTL;

// LLVM
#include <llvm/Module.h>
#include <llvm/Bitcode/ReaderWriter.h>
#include <llvm/System/Path.h>
#include <llvm/ModuleProvider.h>
#include <llvm/System/DynamicLibrary.h>

struct Module::Private {
  Private() : moduleData(0), moduleProvider(0), isStandardLibrary(false) {}
  GTLCore::String name;
  GTLCore::String nameSpace;
  GTLCore::String source;
  bool compiled;
  GTLCore::ModuleData* moduleData;
  llvm::ModuleProvider* moduleProvider;
  std::list<GTLCore::ErrorMessage> compilationErrors;
  bool isStandardLibrary;
};

Module::Module(const GTLCore::String& name) : d(new Private)
{
  d->compiled = false;
  d->name = llvm::sys::Path(name).getBasename();
}

Module::Module() : d(new Private)
{
  d->compiled = false;
  d->name = "";
}

Module::~Module()
{
  if(d->moduleProvider)
  {
    GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleProvider);
    delete d->moduleProvider;
  }
  delete d->moduleData;
  delete d;
}

const GTLCore::String& Module::name() const
{
  return d->name;
}

const GTLCore::String& Module::nameSpace() const
{
  return d->nameSpace;
}

void Module::setSource(const GTLCore::String& str)
{
  setSource("noname", str);
}

void Module::setSource(const GTLCore::String& name, const GTLCore::String& source)
{
  d->name = name;
  d->source = source;
}

void Module::loadFromFile(const GTLCore::String& fileName)
{
  d->isStandardLibrary = fileName.endWith( "ctlstdlib.ctl" );
  d->source = "";
  d->name = llvm::sys::Path(fileName).getBasename();
  std::ifstream in;
  in.open(fileName.c_str() );
  if(not in)
  {
    OCTL_DEBUG( "Impossible to open file " << fileName );
    return;
  }
  GTLCore::String str;
  std::getline(in,str);
  while ( in ) {
    d->source += str;
    d->source += "\n";
    std::getline(in,str);
  }
}

const GTLCore::Function* Module::function(const GTLCore::String& name) const
{
  if( d->moduleData )
  {
    const std::list<GTLCore::Function*>* f = d->moduleData->function( nameSpace(), name);
    if(f and f->size() == 1 )
    {
      return f->front();
    }
  }
  return 0;
}

std::list<const GTLCore::Function*> Module::functions(const GTLCore::String& name) const
{
  if( d->moduleData )
  {
    const std::list<GTLCore::Function*>* fs = d->moduleData->function( nameSpace(), name);
    if(fs )
    {
      std::list<const GTLCore::Function*> fsr;
      foreach(const GTLCore::Function* f, *fs)
      {
        fsr.push_back(f);
      }
      return fsr;
    }
  }
  return std::list<const GTLCore::Function*>();
}

const GTLCore::Function* Module::function(const GTLCore::String& name, std::vector<GTLCore::Parameter>& arguments) const
{
  return d->moduleData->function(d->nameSpace, name, arguments);
}

void Module::compile()
{
  if(d->source.empty()) return;
  if(d->moduleProvider)
  {
    GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleProvider);
    delete d->moduleProvider;
  }
  delete d->moduleData;
  llvm::LLVMContext& context = llvm::getGlobalContext(); // TODO someday (llvm 2.7 ?) use own context
  d->moduleData = new GTLCore::ModuleData(new llvm::Module(d->name, context));

  Compiler c;
  bool result = c.compile(d->isStandardLibrary, d->source, d->name, d->moduleData );

  if(result)
  {
    d->compiled = true;
    llvm::sys::DynamicLibrary::LoadLibraryPermanently( _OPENCTL_LIB_, 0 ); // This needed because when OpenCTL is used in a plugins, OpenCTL symbols aren't loaded globally and then llvm can't find them
    d->moduleData->doLink();
    d->moduleProvider = new llvm::ExistingModuleProvider( d->moduleData->llvmLinkedModule() );
    GTLCore::VirtualMachine::instance()->registerModule( d->moduleProvider );
  } else {
    delete d->moduleData;
    d->moduleData = 0;
    d->compilationErrors = c.errorMessages();
  }
}

GTLCore::String Module::compilationErrorsMessage() const
{
  std::ostringstream os;
  for( std::list<GTLCore::ErrorMessage>::iterator it = d->compilationErrors.begin();
       it != d->compilationErrors.end(); ++it)
  {
    os << it->fileName() << " at " << it->line() << " : " << it->errorMessage()  << std::endl;
  }
  return os.str();
}


bool Module::isCompiled() const
{
  return d->compiled;
}

const std::list<GTLCore::ErrorMessage>& Module::compilationErrors() const
{
  return d->compilationErrors;
}

GTLCore::String Module::asmSourceCode() const
{
  std::ostringstream os;
  os << *d->moduleData->llvmLinkedModule() << std::endl;
  return os.str();
}

std::list<GTLCore::Function*> Module::functions()
{
  return d->moduleData->functions();
}

const GTLCore::ModuleData* Module::data() const
{
  return d->moduleData;
}

const GTLCore::TypesManager* Module::typesManager() const
{
  return d->moduleData->typesManager();
}

==> opengtl_0.9.14.orig.tar.bz2: OpenGTL-0.9.14/OpenCTL/OpenCTL/Module.cpp <==
/*
 *  Copyright (c) 2007 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "Module.h"

#include <sstream>
#include <fstream>

#include "GTLCore/CompilationMessage.h"
#include "GTLCore/CompilationMessages.h"
#include "GTLCore/Function.h"
#include "GTLCore/Parameter.h"
#include "GTLCore/Macros_p.h"
#include "GTLCore/VirtualMachine_p.h"
#include "GTLCore/ModuleData_p.h"
#include "GTLCore/Type.h"

#include "Compiler.h"
#include "Debug.h"

using namespace OpenCTL;

// LLVM
#include <llvm/Module.h>
#include <llvm/Bitcode/ReaderWriter.h>
#include <llvm/System/Path.h>
#include <llvm/Module.h>
#include <llvm/System/DynamicLibrary.h>
#include <llvm/LLVMContext.h>

struct Module::Private {
  Private() : moduleData(0), isStandardLibrary(false) {}
  GTLCore::String name;
  GTLCore::String nameSpace;
  GTLCore::String source;
  bool compiled;
  GTLCore::ModuleData* moduleData;
  GTLCore::CompilationMessages compilationErrors;
  bool isStandardLibrary;
};

Module::Module() : d(new Private)
{
  d->compiled = false;
  d->name = "";
}

Module::~Module()
{
  if(d->moduleData and d->moduleData->llvmLinkedModule())
  {
    GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleData->llvmLinkedModule() );
  }
  delete d->moduleData;
  delete d;
}

GTLCore::String Module::name() const
{
  return d->name;
}

GTLCore::String Module::nameSpace() const
{
  return d->nameSpace;
}

void Module::setSource(const GTLCore::String& name, const GTLCore::String& source)
{
  d->name = name;
  d->source = source;
}

void Module::loadFromFile(const GTLCore::String& fileName)
{
  d->isStandardLibrary = fileName.endWith( "ctlstdlib.ctl" );
  d->source = "";
  d->name = llvm::sys::Path((const std::string&)fileName).getBasename().str();
  std::ifstream in;
  in.open(fileName.c_str() );
  if(not in)
  {
    OCTL_DEBUG( "Impossible to open file " << fileName );
    return;
  }
  std::string str;
  std::getline(in,str);
  while ( in ) {
    d->source += str;
    d->source += "\n";
    std::getline(in,str);
  }
}

std::list<const GTLCore::Function*> Module::functions(const GTLCore::String& name) const
{
  if( d->moduleData )
  {
    const std::list<GTLCore::Function*>* fs = d->moduleData->function( nameSpace(), name);
    if(fs )
    {
      std::list<const GTLCore::Function*> fsr;
      foreach(const GTLCore::Function* f, *fs)
      {
        fsr.push_back(f);
      }
      return fsr;
    }
  }
  return std::list<const GTLCore::Function*>();
}

const GTLCore::Function* Module::function(const GTLCore::String& name, std::vector<GTLCore::Parameter>& arguments) const
{
  return d->moduleData->function(d->nameSpace, name, arguments);
}

void Module::compile()
{
  if(d->source.isEmpty()) return;
  if(d->moduleData and d->moduleData->llvmLinkedModule())
  {
    GTLCore::VirtualMachine::instance()->unregisterModule( d->moduleData->llvmLinkedModule());
  }
  delete d->moduleData;
  llvm::LLVMContext& context = llvm::getGlobalContext(); // TODO someday (llvm 2.7 ?) use own context
  llvm::Module* llvmModule = new llvm::Module((const std::string&)d->name, context);
  d->moduleData = new GTLCore::ModuleData(llvmModule);

  Compiler c;
  bool result = c.compile(d->isStandardLibrary, d->source, d->name, d->moduleData, llvmModule );

  if(result)
  {
    d->compiled = true;
    llvm::sys::DynamicLibrary::LoadLibraryPermanently( _OPENCTL_LIB_, 0 ); // This needed because when OpenCTL is used in a plugins, OpenCTL symbols aren't loaded globally and then llvm can't find them
    d->moduleData->doLink();
    GTLCore::VirtualMachine::instance()->registerModule( d->moduleData->llvmLinkedModule() );
  } else {
    delete d->moduleData;
    d->moduleData = 0;
    d->compilationErrors = c.errorMessages();
  }
}

bool Module::isCompiled() const
{
  return d->compiled;
}

GTLCore::CompilationMessages Module::compilationMessages() const
{
  return d->compilationErrors;
}

GTLCore::String Module::asmSourceCode() const
{
  std::ostringstream os;
  os << *d->moduleData->llvmLinkedModule() << std::endl;
  return os.str();
}

std::list<GTLCore::Function*> Module::functions()
{
  return d->moduleData->functions();
}

const GTLCore::ModuleData* Module::data() const
{
  return d->moduleData;
}

const GTLCore::TypesManager* Module::typesManager() const
{
  return d->moduleData->typesManager();
}
